// file: status_disp.cpp
//
// Subscribe to the datastream status from the relayboard
//  and display on stdout.
//
// compile with:
//  $ gcc -o status_disp status_disp.cpp -llcm
//
// On a system with pkg-config, you can also use:
//  $ gcc -o status_disp status_disp.cpp `pkg-config --cflags --libs lcm`

#include <cstdio>

#include <lcm/lcm-cpp.hpp>
#include "dataspeed_pds_lcm/status_t.hpp"

enum {
    RELAY_STATUS_OFF = 0,
    RELAY_STATUS_ON,
    RELAY_STATUS_NO_LOAD,
    RELAY_STATUS_BAD_RELAY,
    RELAY_STATUS_BAD_FUSE,
    RELAY_STATUS_OVER_CURRENT,
};

enum ScriptType {
    SCRIPT_NONE = 0,
    SCRIPT_IGNITION_ON,
    SCRIPT_IGNITION_OFF,
    SCRIPT_VALET_IGNITION_ON,
    SCRIPT_VALET_IGNITION_OFF,
};

enum {
    MODE_AUTO = 0,
    MODE_MANUAL,
    MODE_VALET,
};

enum {
    ID_MASTER = 0,
    ID_SLAVE1,
    ID_SLAVE2,
    ID_SLAVE3,
};

struct ChannelStatus {
    uint8_t request : 1;
    uint8_t relay : 1;
    uint8_t fuse : 1;
    uint8_t red_led : 1;
    uint8_t status : 4;
};

struct InverterStatus {
    uint8_t request : 1;
    uint8_t status : 1;
    uint8_t overload : 1;
    uint8_t overtemp : 1;
    uint8_t : 4;
};

struct BluetoothStatus {
    uint8_t present : 1;
    uint8_t connected : 1;
    uint8_t enabled : 1;
    uint8_t searching : 1;
    uint8_t : 4;
};

struct FanStatus {
    uint8_t fan1_on : 1;
    uint8_t fan2_on : 1;
    uint8_t : 6;
};

class Handler
{
    public:
        ~Handler() {}

        std::string getStatusText(int status)
        {
            int chan_status = status >> 4;
            switch (chan_status)
            {
                case RELAY_STATUS_OFF: return "off";
                case RELAY_STATUS_ON: return "on";
                case RELAY_STATUS_NO_LOAD: return "no load";
                case RELAY_STATUS_BAD_RELAY: return "bad relay";
                case RELAY_STATUS_BAD_FUSE: return "bad fuse";
                case RELAY_STATUS_OVER_CURRENT: return "over current";
            }
            return "";
        }

        std::string getModeText(int mode)
        {
            switch (mode)
            {
                case MODE_AUTO: return "auto";
                case MODE_MANUAL: return "manual";
                case MODE_VALET: return "valet";
            }
            return "";
        }

        std::string getScriptText(int script)
        {
            switch (script)
            {
                case SCRIPT_NONE: return "none";
                case SCRIPT_IGNITION_ON: return "startup";
                case SCRIPT_IGNITION_OFF: return "shutdown";
                case SCRIPT_VALET_IGNITION_ON: return "valet startup";
                case SCRIPT_VALET_IGNITION_OFF: return "valet shutdown";
            }
            return "";
        }

        std::string getIdText(int id)
        {
            switch (id)
            {
                case ID_MASTER: return "Master";
                case ID_SLAVE1: return "Slave 1";
                case ID_SLAVE2: return "Slave 2";
                case ID_SLAVE3: return "Slave 3";
            }
            return "";
        }

        std::string getInverterRequestText(int status)
        {
            InverterStatus* state = (InverterStatus*)&status;
            if (state->request)
                return "on";
            else
                return "off";
            return "";
        }

        std::string getInverterStatusText(int status)
        {
            InverterStatus* state = (InverterStatus*)&status;
            if (state->status)
                return "on";
            else
                return "off";
            return "";
        }

        std::string getInverterOverloadText(int status)
        {
            InverterStatus* state = (InverterStatus*)&status;
            if (state->overload)
                return "yes";
            else
                return "no";
            return "";
        }

        std::string getInverterOvertempText(int status)
        {
            InverterStatus* state = (InverterStatus*)&status;
            if (state->overtemp)
                return "yes";
            else
                return "no";
            return "";
        }

        void onStatusMessage(const lcm::ReceiveBuffer* rbuf,
                const std::string& chan,
                const dataspeed_pds_lcm::status_t* msg)
        {
            printf("\nStatus of %s:\n", getIdText(msg->unit_id).c_str());
            printf("  Voltage             = %.2f V\n", msg->voltage);
            printf("  Ignition1           = %.2f V\n", msg->ignition[0]);
            printf("  Ignition2           = %.2f V\n", msg->ignition[1]);
            printf("  Ignition3           = %.2f V\n", msg->ignition[2]);
            printf("  Internal temp 1     = %.2f °C\n", msg->board_temp[0]);
            printf("  Internal temp 2     = %.2f °C\n", msg->board_temp[1]);
            printf("  Internal temp 3     = %.2f °C\n", msg->board_temp[2]);
            printf("  External temp 1     = %.2f °C\n", msg->thermocouple[0]);
            printf("  External temp 2     = %.2f °C\n", msg->thermocouple[1]);
            printf("  External temp 3     = %.2f °C\n", msg->thermocouple[2]);
            printf("  External temp 4     = %.2f °C\n", msg->thermocouple[3]);
            printf("  Total current       = %.2f A\n", msg->total_current);
            printf("  Mode                = %s\n", getModeText(msg->mode).c_str());
            printf("  Script              = %s\n", getScriptText(msg->script).c_str());
            printf("  Inverter request    = %s\n", getInverterRequestText(msg->inverter_status).c_str());
            printf("  Inverter status     = %s\n", getInverterStatusText(msg->inverter_status).c_str());
            printf("  Inverter overload   = %s\n", getInverterOverloadText(msg->inverter_status).c_str());
            printf("  Inverter overtemp   = %s\n", getInverterOvertempText(msg->inverter_status).c_str());
            printf("  Bluetooth present   = %s\n", ((BluetoothStatus*)&msg->bluetooth_status)->present ? "yes" : "no");
            printf("  Bluetooth connected = %s\n", ((BluetoothStatus*)&msg->bluetooth_status)->connected ? "yes" : "no");
            printf("  Bluetooth enabled   = %s\n", ((BluetoothStatus*)&msg->bluetooth_status)->enabled ? "yes" : "no");
            printf("  Bluetooth searching = %s\n", ((BluetoothStatus*)&msg->bluetooth_status)->searching ? "yes" : "no");
            printf("  Internal fan 1      = %s\n", ((FanStatus*)&msg->fan_status)->fan1_on ? "on" : "off");
            printf("  Internal fan 2      = %s\n", ((FanStatus*)&msg->fan_status)->fan2_on ? "on" : "off");
            for(int i = 0; i < 12; i++)
            {
                printf("  Current%2d           = %.3f A\n", i+1, msg->current[i]);
                printf("   Status%2d           = %s\n",   i+1, getStatusText(msg->status[i]).c_str());
            }
        }
};

int main(int argc, char** argv)
{
    lcm::LCM lcm("udpm://225.0.0.0:7667?ttl=1");

    if(!lcm.good())
        return 1;

    Handler handlerObject;
    lcm.subscribe("STATUS", &Handler::onStatusMessage, &handlerObject);

    while(0 == lcm.handle());

    return 0;
}
